package de.elisabetheckstaedt.dipl.common;

import java.util.Arrays;

import de.elisabetheckstaedt.dipl.real.AbKM;
import de.elisabetheckstaedt.dipl.real.Kaeltemaschine;
import de.elisabetheckstaedt.dipl.real.Kompaktanlage;
import de.elisabetheckstaedt.dipl.real.Rueckkuehler;
import de.elisabetheckstaedt.dipl.real.Stoff;
import de.elisabetheckstaedt.dipl.real.Turboverdichter;
import de.elisabetheckstaedt.dipl.real.Verbraucher;
import de.elisabetheckstaedt.dipl.real.Verdunstungsrueckkuehler;

/**
 * fasst Verbraucher und Klimadatensatz zusammen
 * @author Elisabeth Eckstdt
 *
 */
public class Versorgungsaufgabe {
	
	private Verbraucher[] Verbraucher;
	private Klimadatensatz Klimadatensatz;
	
	public Versorgungsaufgabe(Verbraucher[] verbraucher, Klimadatensatz klimadatensatz) {
		super();
		Verbraucher = verbraucher;
		Klimadatensatz = klimadatensatz;
	}

	public Versorgungsaufgabe(Klimadatensatz klimadatensatz) {
		super();
		Klimadatensatz = klimadatensatz;
	}

	public Versorgungsaufgabe(Verbraucher[] verbraucher) {
		super();
		Verbraucher = verbraucher;
	}

	public Versorgungsaufgabe() {
		super();
	}
	
	/**
	 * Summe ber alle Verbraucher
	 * @return 8760 langer Vektor mit summierter Klteleistung der Versorgungsaufgabe
	 * liest Klimadatensatz.Aussentemperatur
	 * ruft Verbraucher.Kaelteleistung()
	 */
	public double[] Lastgang() //in kW
	{
		double[] Kaelteleistungssumme = new double[8760];
		Arrays.fill(Kaelteleistungssumme, 0);
		for (int i=0; i<Verbraucher.length; i++)
		{
			double[] KLi = Verbraucher[i].Kaelteleistung(Klimadatensatz.getAussentemperatur());
			for (int j=0; j<8760; j++)
			{
				Kaelteleistungssumme[j] = Kaelteleistungssumme[j] + KLi[j];
			}
		}
		return Kaelteleistungssumme;
	}
	
	/**
	 * Summe ber alle Verbraucher, absteigend sortiert
	 * @return 8760 langer Vektor mit summierter Klteleistung der Versorgungsaufgabe
	 * ruft Lastgang(), Helper.reverse()
	 */
	public double[] Lastgang_geordnet() //in kW
	{
		double[] array = Lastgang();
		Arrays.sort(array); //aufsteigend
		return Helper.reverse(array);
	}
	
	/**
	 * @return [kW]
	 */
	public double Spitzenlast()
	{
		return Lastgang_geordnet()[0];
	}
	
	/**
	 * liest Auentemperatur
	 * ruft Verbraucher.Klteleistuung
	 * ruft Verbraucher.Massenstrom
	 * @param Kaeltetraeger
	 * @param Vorlauftemperatur
	 * @return summiert die Massenstrme alle Verbraucher
	 */
	public double[] Massenstrom(Stoff Kaeltetraeger, double Vorlauftemperatur)
	{
		double[] Massenstrom = new double[8760];
		Arrays.fill(Massenstrom, 0);
		double[] T_A = Klimadatensatz.getAussentemperatur();
		for (int i=0; i<Verbraucher.length; i++)
		{
			double[] mp = Verbraucher[i].Massenstrom(Kaeltetraeger, 
					Verbraucher[i].Kaelteleistung(T_A), 
					Vorlauftemperatur, T_A);
			for (int j=0; j<8760; j++)
			{
				Massenstrom[j] = Massenstrom[j] + mp[j];
			}
		}
		return Massenstrom;
	}
	
	/**
	 * @return Volumenstrom des Subnetzes in m/s
	 * liest KT.Dichte
	 * ruft VA.Massenstrom
	 */
	public double[] Volumenstrom(Stoff Kaeltetraeger, double Vorlauftemperatur)
	{
		double[] mp = Massenstrom(Kaeltetraeger, Vorlauftemperatur);
		double rho = Kaeltetraeger.Dichte(10);
		double[] Vp = new double[8760];
		for (int i=0; i<8760; i++)
		{
			Vp[i] = mp[i] / (rho*1000); 
		}
		return Vp;
	}
	
	/**
	 * @param Kaeltetraeger
	 * @param Vorlauftemperatur
	 * @return Rcklauftemperatur [C]
	 * mittelt die Rcklauftemperaturen aller Verbraucher (gewichtet ber den Massenstrom der einzelnen Verbraucher)
	 * liest Aussentemperaturen
	 * ruft Verbraucher.Rcklauftemperatur, Verbraucher.Massenstrom
	 */
	public double[] Ruecklauftemperatur(Stoff Kaeltetraeger, double Vorlauftemperatur)
	{
//		1.	Jeder Verbraucher hat t_RL(t_A)
//		2.	Damit hat jeder Verbraucher ein delta_T (da Vorlauftemperatur von der Zentrale vorgebeen) und damit einen Massenstrom
//		3.	Jede Versorgungsaufgabe (=Ansammlung von Verbrauchern) hat damit einen Massenstrom, der sich aus der Summe der Teilstrme ergibt
//		4.	Jede Versorgungsaufgabe hat eine konstante Vorlauftemperatur und eine Rcklauftemperatur, die sich aus dem gewichteten Mittel (mit Hilfe der Masenstromanteile) der Teilstrme zusammensetzt (da Summe der Wrmestrme = Gesamtwrmestrom (kein Verluste))

		double[] Ruecklauftemperatur = new double[8760];
		double[][] RLT = new double[Verbraucher.length][8760];
		double[][] mp = new double[Verbraucher.length][8760];
		double[] T_A = Klimadatensatz.getAussentemperatur();
		for (int i=0; i<Verbraucher.length; i++)
		{
			RLT[i] = Verbraucher[i].Ruecklauftemperatur(T_A);
			mp[i] = Verbraucher[i].Massenstrom(Kaeltetraeger, Verbraucher[i].Kaelteleistung(T_A), Vorlauftemperatur, T_A);
		}
		double[] mpges = Massenstrom(Kaeltetraeger, Vorlauftemperatur);
		double TMP;
		for (int j=0; j<8760; j++)
		{
			TMP = 0;
			for (int k=0; k<Verbraucher.length; k++)
			{
				TMP = TMP + RLT[k][j] * mp[k][j];
			}
			Ruecklauftemperatur[j] = TMP / mpges[j];
		}
		return Ruecklauftemperatur;
	}
	
	/**
	 * @return 1. Zeile X, 2. Zeile Y; Spalten sind die unterschiedlichen Verbraucher
	 * [m]
	 */
	public double[][] Koordinaten()
	{
		//Koordinaten aller Verbraucher einer Versorgungsaufgabe in zweizeiliger Form
		double[][] Koordinaten = new double[2][Verbraucher.length];
		for (int i=0; i<Verbraucher.length; i++)
		{
			Koordinaten[0][i] = Verbraucher[i].getOrt(0);
			Koordinaten[1][i] = Verbraucher[i].getOrt(1);
		}
		return Koordinaten;
	}

	/**
	 * legt die Gre der ntigen Kltemaschinen fest (80% der Zeit muss es die GlM alleine schaffen)
	 * @param kznummer
	 * @return {Grundlastmaschine, Spitzenlastmaschine, Nennleistung GLM, SLM, Leistungsanteile[][]}
	 */
	public Object[] maschinenauswahl_auto(int kznummer)
	{
		double spitzenlast = Lastgang_geordnet()[0];
		double einzelmaschinengrenze = 500; //FIXME einzelmaschinengrenze
		Object[] rgw = null;
		if (spitzenlast == Lastgang_geordnet()[8759]) //eine Maschine AbKM bei konstanter Last
		{
			Rueckkuehler rk = new Verdunstungsrueckkuehler("RK gesamt","offen","axial");
			Kaeltemaschine GLM = new AbKM("AbKM gesamt", rk, "7");
			Kaeltemaschine[] kaeltemaschinen = new Kaeltemaschine[1];
			kaeltemaschinen[0] = GLM;
			double[] maschinenleistungen = new double[1];
			maschinenleistungen[0] = spitzenlast;
			double[][] LA = new double[1][8760];
			double[] LAGlM = new double[8760];
			Arrays.fill(LAGlM, 1);
			LA[0] = LAGlM;
			rgw = new Object[3];
			rgw[0] = kaeltemaschinen;
			rgw[1] = maschinenleistungen;
			rgw[2] = LA;
		}
		else if (spitzenlast < einzelmaschinengrenze) //eine Kompaktanlage bei kleiner unkonstanter Last
		{
			Kaeltemaschine GLM = new Kompaktanlage("Kompaktanlage","Schraube","C");
			Kaeltemaschine[] kaeltemaschinen = new Kaeltemaschine[1];
			kaeltemaschinen[0] = GLM;
			double[] maschinenleistungen = new double[1];
			maschinenleistungen[0] = spitzenlast;
			double[][] LA = new double[1][8760];
			double[] LAGlM = new double[8760];
			Arrays.fill(LAGlM, 1);
			LA[0] = LAGlM;
			rgw = new Object[3];
			rgw[0] = kaeltemaschinen;
			rgw[1] = maschinenleistungen;
			rgw[2] = LA;
		}

		else //2 Maschinen
		{
			String[] name = new String[2];  //TODO bisher maximal 2 KM im Auswahlfenster (auch nchste Zeile)
			double[] maschinenleistungen = new double[2];
			int laufzeit = new Double(100 - 80 /100 * 8760).intValue();
			maschinenleistungen[0] = Lastgang_geordnet()[laufzeit];
			double[][] LA;
			LA = new double[2][8760];
			name[0] = "Grundlastmaschine Teilsystem " + kznummer;
			maschinenleistungen[1] = spitzenlast - maschinenleistungen[0];
			name[1] = "Spitzenlastmaschine Teilsystem " + kznummer;
			double[] LAGlM = new double[8760];
			double[] LASlM = new double[8760];
			double[] kaelteleistung = Lastgang();
			for (int i=0; i<8760; i++) //TODO Speicherentladung hat auch einen Leistungsanteil
			{
				if (kaelteleistung[i]<maschinenleistungen[0])
				{
					LASlM[i] = 0;
					LAGlM[i] = 1;
				}
				else
				{
					LAGlM[i] = maschinenleistungen[0]/kaelteleistung[i];
					LASlM[i] = 1 - LAGlM[i];
				}
			}
			LA[0] = LAGlM;
			LA[1] = LASlM;
			Rueckkuehler rk = new Verdunstungsrueckkuehler("RK " + name[0],"offen","axial");
			Kaeltemaschine GLM = new AbKM(name[0], rk, "7");
			rk = new Verdunstungsrueckkuehler("RK " + name[1],"offen","axial");
			Kaeltemaschine SLM = new Turboverdichter(name[1], rk, "6");
			Kaeltemaschine[] kaeltemaschinen = new Kaeltemaschine[2];
			kaeltemaschinen[0] = GLM;
			kaeltemaschinen[1] = SLM;
			rgw = new Object[3];
			rgw[0] = kaeltemaschinen;
			rgw[1] = maschinenleistungen;
			rgw[2] = LA;
		}
		return rgw;
	}
	public void setVerbraucher(Verbraucher[] verbraucher) {
		Verbraucher = verbraucher;
	}
	public Klimadatensatz getKlimadatensatz() {
		return Klimadatensatz;
	}
	public Verbraucher[] getVerbraucher() {
		return Verbraucher;
	}
}
