package de.elisabetheckstaedt.dipl.real;

import de.elisabetheckstaedt.dipl.common.Helper;
import de.elisabetheckstaedt.dipl.common.Klimadatensatz;

/**
 * abstrakte Vaterklasse fr alle Kltemaschinen
 * @author Elisabeth Eckstdt
 */
public abstract class Kaeltemaschine extends Stromverbraucher
{
	/**
	 * [kW]
	 */
	protected double Nennleistung;
	
	/**
	 * Definiert den Anteil an bentigter Hilfsenergie bezogen auf die bereitgestellte Klteleistung.<p>
	 * P_hilfs/Q_0<p>
	 * [kWh_Strom/kWh_Kaelte]<p>
	 * Wird, wenn vom Benutzer nichts anderes eingeben, in den Kindklassen entsprechend DIN auf 0 gesetzt.<p>
	 */
	protected double Hilfsantriebe;
	
	/**
	 * [m/kWh] Wasser/Klte
	 */
	protected double Wasserfaktor;
	
	/**
	 * Wasser oder Luft
	 */
	protected final String Kuehlmedium;
	
	/**
	 * nach DIN V 18599
	 */
	protected final String Teillastregelung;
	
	/**
	 * der Maschine zugeordneter Rckkhler
	 */
	protected final Rueckkuehler Rueckkuehler;
	
	/**
	 * [m]
	 */
	protected double laenge;
	/**
	 * [m]
	 */
	protected double breite;
	/**
	 * [m]
	 */
	protected double hoehe;

	/**
	 * setzt Kuehlmedium je nach Rckkhlertyp
	 * @param Name
	 * @param rk
	 * @param teillastregelung
	 */
	protected Kaeltemaschine(String Name, Rueckkuehler rk, String teillastregelung) {
		super(Name);
		Rueckkuehler = rk;
		this.Teillastregelung = teillastregelung;
		if (rk instanceof Wasserrueckkuehler)
		{
			Kuehlmedium = "Wasser";
		}
		else
		{
			Kuehlmedium = "Luft";
		}
	}
	
	/**
	 * @param nennleistung
	 * @param t_K_A
	 */
	public abstract void init(double nennleistung, double t_K_A);
	
	protected abstract void Investition();

	/**
	 * ruft Stromverbrauch() und RK.Stromverbrauch()
	 * @param Kaelteleistung
	 * @param kds
	 */
	public void stromGes(double[] Kaelteleistung, Klimadatensatz kds)
	{
		Stromverbrauch(Kaelteleistung,kds.getAussentemperatur());
		Rueckkuehler.Stromverbrauch(Rueckkuehlleistung(Kaelteleistung), kds);
	}

	/**
	 * nach DIN V 18599 + Hilfsantriebe entspechend Angaben in Machbarkeitsstudie
	 * @param Kaelteleistung in kWh (=kW) fr jede Stunde des Jahres
	 * @param Auenlufttemperatur
	 * @return Stromverbrauch in kWh (=kW) fr jede Stunde des Jahres
	 */
	public abstract void Stromverbrauch(double[] Kaelteleistung, double[] t_L);
	
	/**
	 * @param Kaelteleistung
	 * @return Rckkhlleistung [kW] fr jede Jahresstunde
	 */
	public abstract double[] Rueckkuehlleistung (double[] Kaelteleistung);
	
	/**
	 * liest {@link Kaeltemaschine#Kuehlmedium}, {@link Kaeltemaschine#Teillastregelung}
	 * @param Belastung ist Aussentemperatur bei Luftgekhlten Maschinen, ist Klteleistung bei Wassergekhlten
	 * @return Part Load Value entsprechend DIN V 18599-7 fr jede Jahresstunde, was in der DIN nicht so vorgesehen ist
	 */
	public double[] PLV (double[] Belastung)
	{
		double[] PLV = new double[8760];
		if (Kuehlmedium == "Luft")
		{
			double[] Aussentemperatur = Belastung;
			double T_A;
			double k_n; //Teillaststufe
			for (int i=0; i<8760; i++)
			{
				T_A = Aussentemperatur[i];
				if (T_A <= 12) k_n = 0.1;
				else if (T_A <= 14.2) k_n = 0.2;
				else if (T_A <= 16.5) k_n = 0.3;
				else if (T_A <= 18.7) k_n = 0.4;
				else if (T_A <= 20.9) k_n = 0.5;
				else if (T_A <= 23.1) k_n = 0.6;
				else if (T_A <= 25.3) k_n = 0.7;
				else if (T_A <= 27.6) k_n = 0.8;
				else if (T_A <= 29.8) k_n = 0.9;
				else k_n = 1;
				if (Teillastregelung == "A")
				{
					if (k_n == 0.1) PLV[i] = 1.52;
					else if (k_n == 0.2) PLV[i] = 1.49;
					else if (k_n == 0.3) PLV[i] = 1.45;
					else if (k_n == 0.4) PLV[i] = 1.40;
					else if (k_n == 0.5) PLV[i] = 1.35;
					else if (k_n == 0.6) PLV[i] = 1.29;
					else if (k_n == 0.7) PLV[i] = 1.22;
					else if (k_n == 0.8) PLV[i] = 1.15;
					else if (k_n == 0.9) PLV[i] = 1.08;
					else PLV[i] = 1.00;
				}
				else if (Teillastregelung == "B")
				{
					if (k_n == 0.1) PLV[i] = 1.60;
					else if (k_n == 0.2) PLV[i] = 1.75;
					else if (k_n == 0.3) PLV[i] = 1.66;
					else if (k_n == 0.4) PLV[i] = 1.56;
					else if (k_n == 0.5) PLV[i] = 1.42;
					else if (k_n == 0.6) PLV[i] = 1.40;
					else if (k_n == 0.7) PLV[i] = 1.32;
					else if (k_n == 0.8) PLV[i] = 1.21;
					else if (k_n == 0.9) PLV[i] = 1.10;
					else PLV[i] = 1.00;
				}
				else if (Teillastregelung == "C")
				{
					if (k_n == 0.1) PLV[i] = 0.95;
					else if (k_n == 0.2) PLV[i] = 0.96;
					else if (k_n == 0.3) PLV[i] = 1.15;
					else if (k_n == 0.4) PLV[i] = 1.29;
					else if (k_n == 0.5) PLV[i] = 1.32;
					else if (k_n == 0.6) PLV[i] = 1.06;
					else if (k_n == 0.7) PLV[i] = 1.09;
					else if (k_n == 0.8) PLV[i] = 1.08;
					else if (k_n == 0.9) PLV[i] = 1.04;
					else PLV[i] = 1.00;
				}
				else
					System.err.println("Diese Art der Teillastregelung ist fr eine luftgekhlte Maschine nicht " +
							"vorgesehen (Kaeltemaschine.Teillastfaktor(Versorgungsaufgabe)) Teillastregelung: " + Teillastregelung);
				}
			}
		else if (Kuehlmedium == "Wasser")
		{
			double[] Kaeltelast = Belastung;
			double[] t_w_a = ((Wasserrueckkuehler) Rueckkuehler).t_W_A();
			for (int i=0; i<8760; i++)
			{
				double f_1n = f_1i(Kaeltelast[i]);
				double f_2n = f_2i(t_w_a[i]);
				PLV[i] = f_1n * f_2n;
			}
		}
		else
			System.err.println("Fr dieses Khlmedium (" + Kuehlmedium + ") sind keine Teillastwerte hinterlegt");
		return PLV;
	}
	
	/**
	 * @param Kaeltelast
	 * @return Teillastfaktor Kltemaschine
	 */
	protected double f_1i(double Kaeltelast)
	{
		double f_1n = 0;
		double Auslastung = Kaeltelast / Nennleistung;
		double k_n = Math.ceil(Auslastung*10)/10;
		//TODO weitere Teillastregelungen wassergekhlt implementieren
		if (Teillastregelung == "5") //Schraubenverdichter
		{
			if (k_n == 0.1) f_1n = 0.45;
			else if (k_n == 0.2) f_1n = 0.67;
			else if (k_n == 0.3) f_1n = 0.80;
			else if (k_n == 0.4) f_1n = 0.91;
			else if (k_n == 0.5) f_1n = 0.99;
			else if (k_n == 0.6) f_1n = 1.03;
			else if (k_n == 0.7) f_1n = 1.05;
			else if (k_n == 0.8) f_1n = 1.04;
			else if (k_n == 0.9) f_1n = 1.02;
			else f_1n = 1.00;
		}
		else if (Teillastregelung == "6") //Turboverdichter
		{
			if (k_n == 0.1) f_1n = 0.50;
			else if (k_n == 0.2) f_1n = 0.72;
			else if (k_n == 0.3) f_1n = 0.84;
			else if (k_n == 0.4) f_1n = 0.93;
			else if (k_n == 0.5) f_1n = 0.99;
			else if (k_n == 0.6) f_1n = 1.01;
			else if (k_n == 0.7) f_1n = 1.03;
			else if (k_n == 0.8) f_1n = 1.02;
			else if (k_n == 0.9) f_1n = 1.01;
			else f_1n = 1.00;
		}
		else if (Teillastregelung == "7") //fr AbKM
		{
			if (k_n == 0.1) f_1n = 0.61;
			else if (k_n == 0.2) f_1n = 0.88;
			else if (k_n == 0.3) f_1n = 1.00;
			else if (k_n == 0.4) f_1n = 1.06;
			else if (k_n == 0.5) f_1n = 1.06;
			else if (k_n == 0.6) f_1n = 1.05;
			else if (k_n == 0.7) f_1n = 1.04;
			else if (k_n == 0.8) f_1n = 1.03;
			else if (k_n == 0.9) f_1n = 1.01;
			else f_1n = 1.00;
		}
		else
			System.err.println("Fr diese Art der Teillastregelung sind keine Teillastfaktoren hinterlegt " +
					"(Kaeltemaschine.f1_i(Kaeltelast) Teillastregelung: " + Teillastregelung);
		return f_1n;
		
	}
	
	/**
	 * @param t_w_a Khlwasseraustrittstemperatur [C]
	 * @return Teillastfaktor der Rckkhlung (bzgl. KM-Strom) in dem Zeitschritt mit dieser Khlwasseraustrittstemperatur
	 */
	protected abstract double f_2i(double t_w_a);
	
	/**
	 * setzt Wasserverbrauch von Verdunstungsrckkhlern
	 * @param Kaelteleistung in kWh (=kW) fr jede Stunde des Jahres
	 */
	public void Wasserverbrauch(double[] Kaelteleistung)
	{
		if (Rueckkuehler instanceof Verdunstungsrueckkuehler)
		{
			for (int i=0; i<8760; i++)
			{
				((Verdunstungsrueckkuehler) Rueckkuehler).Wasserverbrauch[i] = Kaelteleistung[i] * Wasserfaktor;
			}
		}
	}

	public void setHilfsantriebe(double Standardwert) {
			Hilfsantriebe = Standardwert;
	}

	public void setWasserfaktor(double Standardwert) {
			Wasserfaktor = Standardwert;
	}

	public abstract void setNennleistung(double nennleistung);
	
	public String Investitionstext()
	{
		return super.Investitionstext() + " Nennleistung: " + Helper.Zahlformat(Nennleistung,0) + " kW";
	}
	public double getBreite() {
		return breite;
	}
	public double getHoehe() {
		return hoehe;
	}
	public double getLaenge() {
		return laenge;
	}

	public double getNennleistung() {
		return Nennleistung;
	}

	public double getHilfsantriebe() {
		return Hilfsantriebe;
	}

	public double getWasserfaktor() {
		return Wasserfaktor;
	}

	public String getKuehlmedium() {
		return Kuehlmedium;
	}

	public String getTeillastregelung() {
		return Teillastregelung;
	}

	public Rueckkuehler getRueckkuehler() {
		return Rueckkuehler;
	}
}
